#!/usr/bin/env python
"""
U2FsdGVkX19cV2plTMSqbpdFaJuEChbUka1NX+X4qeSMMFuTquH1Wd+vL6e2Hf4H
CAvb+R3+MDOCjdOyhgS3DNleCDqZRRJorq7lehTQzbA/pY9TMQBMyUnm63wUOjd/
EKVAggOM+jODpiFn0+FN4le8LUoOErVSZSR7L4gka/7L69wT82r90gq12H1mFNns
judHXORdh3Iud6J8UQHqSZUSFRfnd1IetIIySIIVxxCPE0DY2CS7Y/TjmEiHduzm
l4TiHrn9MlQOJY21PIjswrB9EIS/fsQsNV09BIBh6cJTu2jPULRjOpVGw+liyvkJ
I6HynL0O7XEVC61l2wiQJA==
"""

import construct

from base_msg import PSBP, SOURCE_ADDRESS
from utils import fmt_repr, containerize, exclude_fields

PSBP_MSG_POWER_W_REQ = 0x17


class MsgPowerWReq(PSBP):
    """
    power W req
    """
    _parser = construct.Struct(
        'set_value' / construct.Int8ub)
    __slots__ = [
        'set_value',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgPowerWReq,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)
            self.from_binary(psbp.payload)
        else:
            super(MsgPowerWReq, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_POWER_W_REQ
            self.set_value = kwargs.pop('set_value')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgPowerWReq._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgPowerWReq._parser.build(c)
        return self.pack()


PSBP_MSG_POWER_W_ACK = 0x17


class MsgPowerWAck(PSBP):
    """
    power W ack
    """
    _parser = construct.Struct(
        'set_value' / construct.Int8ub,
        'actual_value' / construct.Int8ub)
    __slots__ = [
        'set_value',
        'actual_value',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgPowerWAck,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)

            self.from_binary(psbp.payload)
        else:
            super(MsgPowerWAck, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_POWER_W_ACK
            self.set_value = kwargs.pop('set_value')
            self.actual_value = kwargs.pop('actual_value')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgPowerWAck._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgPowerWAck._parser.build(c)
        return self.pack()


PSBP_MSG_POWER_R_REQ = 0x16


class MsgPowerRReq(PSBP):
    """
    power R req
    """

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgPowerRReq,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)
            super.from_binary(psbp.payload)
        else:
            super(MsgPowerRReq, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_POWER_R_REQ

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = super(MsgPowerRReq, self)._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))


PSBP_MSG_POWER_R_ACK = 0x16


class MsgPowerRAck(PSBP):
    """
    power R ack
    """
    _parser = construct.Struct(
        'set_value' / construct.Int8ub,
        'actual_value' / construct.Int8ub)
    __slots__ = [
        'set_value',
        'actual_value',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgPowerRAck, self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                               psbp.command, psbp.payload, psbp.crc)

            self.from_binary(psbp.payload)
        else:
            super(MsgPowerRAck, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_POWER_R_ACK
            self.set_value = kwargs.pop('set_value')
            self.actual_value = kwargs.pop('actual_value')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgPowerRAck._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgPowerRAck._parser.build(c)
        return self.pack()


msg_classes = {
    0x17: MsgPowerWAck,
    0x16: MsgPowerRAck,
}

if __name__ == '__main__':
    print('Simple Test for a Power Messages')
    msg = MsgPowerWReq(receiver_address=0x40, flag=(1 << 4), set_value=100)
    packed_msg = msg.pack()
    print(':'.join(hex(ord(x))[2:] for x in packed_msg))
    raw_bin = msg.to_binary()
    print(':'.join(hex(ord(x))[2:] for x in raw_bin))

    msg = MsgPowerRReq(receiver_address=0x40, flag=(1 << 4))
    packed_msg = msg.pack()
    print(':'.join(hex(ord(x))[2:] for x in packed_msg))
    raw_bin = msg.to_binary()
    print(':'.join(hex(ord(x))[2:] for x in raw_bin))
